/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
#include <QGraphicsWidget>
#include <QGraphicsGridLayout>
#include <QGraphicsTextItem>
#include <QGraphicsProxyWidget>
#include <KMessageBox>
#include <KRun>
//////////////////////////////////////
#include "device.h"
#include "settings.h"
#include "dia_change_nick_name.h"
//////////////////////////////////////

/////////////////////////////////////////////////////////////
/////         																					/////
/////     Class Device																	/////
/////         																					/////
/////////////////////////////////////////////////////////////
Device::Device( const QString& dev, const QString& mt, SettingsStructure *set) : QObject (NULL) 
{
		settings=set;
		current_command="";
		
		set_max_space(1);
		set_free_space(0);
		set_type(HardDisk);
		set_managed(!FALSE);
		set_dev_name(dev);
		set_mount_point(mt);
		set_nick_name(mt);
		set_state(Unmounted);
		set_show_free_space(TRUE);
		set_sudo(FALSE);
				
		command = new KProcess(this);
		command->setOutputChannelMode(KProcess::SeparateChannels);
		command->setNextOpenMode(QIODevice::ReadWrite);
		connect(command,SIGNAL(finished( int, QProcess::ExitStatus )) , this ,SLOT(command_finished( int, QProcess::ExitStatus )));
		connect(command,SIGNAL(error( QProcess::ProcessError )) , this ,SLOT(command_failed( QProcess::ProcessError )));
//  		connect(command,SIGNAL(readyReadStandardOutput()) , this ,SLOT(enter_input()));
		
		
}

//////////////////////////////////
/////   Interactions
////////////////////////////////
void	Device::toggle_mount()
{
		do_mount(state);
}

void Device::launch_file_manager()
{
		KRun *proc = new KRun (KUrl(get_mount_point()),NULL,(mode_t)0,TRUE,TRUE);
}

void Device::do_mount( const MountState &m)
{
		QStringList args;		
		command->terminate();
		current_command="mount";
		
		QString mnt="mount";
		QString umnt="umount";
		
		if (((get_type()==FUSE_FS) || (get_type()==SSH_FS)))
		{			
// 			mnt ="konsole";
// 			umnt="konsole";
// 			args << "-e";
			if (m==Unmounted)
			{
// 				args << "mount";
				mnt = "mount";
			}
			else
			{
// 				args << "fusermount";				
				umnt="fusermount";
				args << "-u";
			}
		}
		
		if (is_sudo())
		{
			if (m == Unmounted)
				args << "-t" << mnt << get_mount_point();
			else
				args << "-t"<< umnt << get_mount_point();		
			command->setProgram("/usr/lib/kde4/libexec/kdesu",args);
			
		}
		else
		{
			args << get_mount_point();
			if (m == Unmounted)
				command->setProgram(mnt,args);
			else
				command->setProgram(umnt,args);
			
		}
		command->start();		
}

void Device::do_load(const bool &m)
{	
		command->terminate();
		
		QStringList args;		
		current_command="load";
		
		QString cur_com;
		if (m)
			cur_com=structure.load_command;
		else
			cur_com=structure.eject_command;		
		
		cur_com=cur_com.arg(get_dev_name());
		
		QStringList command_line = cur_com.split(" ");
		cur_com=command_line.at(0);
		
		int n=1;
		while (n<command_line.count())
			args << command_line.at(n++);
		
		command->setProgram(cur_com,args);		
		command->start();
}


void	Device::command_failed							(QProcess::ProcessError err)
{
		QString S;
		for (int n=0;n<command->program().count();n++)
			S+=command->program().at(n);
		
		switch (err)
		{
			case QProcess::FailedToStart :
				KMessageBox::detailedError(NULL, QString("The requested command ('%1') didn't start").arg(S), command->readAllStandardError() , "Command failed to start" );	
// 				qDebug() << "program didn't start";
				break;
				
			case QProcess::UnknownError :
				KMessageBox::detailedError(NULL, QString("The requested command ('%1') ran into an unknown error. Sorry.").arg(S), command->readAllStandardError() , "Command failed for some reason" );	
// 				qDebug() << " unknown error";
				break;
				
			default :
// 				qDebug() << "error ??";
				break;
		}
		current_command="";
}

void	Device::command_finished (int exitCode, QProcess::ExitStatus exitStatus)
{
		if (exitStatus != QProcess::NormalExit)
		{
			return;
		}
		
		if (exitCode != 0)
		{
			if (current_command=="mount")
				KMessageBox::detailedError(NULL, "Cannot mount/unmount the Device, maybe you don't have the necessary rights", command->readAllStandardError() , "Mount Error" );
			else if (current_command=="load")
				KMessageBox::detailedError(NULL, "Cannot load/eject the Device, maybe this device cannot be affected by the action", command->readAllStandardError() , "Eject Error" );
		}
		else
			emit SIG_refresh_mtab();
		
		current_command="";
		
}


void Device::enter_input()
{
	
}


////////////////////////////////////////////////////
////   Data handling and syncing
////////////////////////////////////////////////////
void Device::some_data_changed()
{ 
	// Make sure that everything is synced properly
	set_info(get_info());
	refresh_free_space_text();	
}

void Device::refresh_free_space_text()
{
	if (!settings->initialized)
		return;
	
	float fs = 1.0*get_free_space();
	float ms = 1.0*get_max_space();
	float us = ms-fs;
	
	Unit u_fs,u_ms,u_us;
	if (settings->preferences.use_best_unit)
	{
		to_best_unit(u_fs,fs);
 		to_best_unit(u_ms,ms);
 		to_best_unit(u_us,us);
	}
	else
	{
		u_fs=settings->preferences.size_unit;
		u_ms=settings->preferences.size_unit;
		u_us=settings->preferences.size_unit;
		to_unit(u_fs,fs);
		to_unit(u_ms,ms);
		to_unit(u_us,us);
	}

	fs_text=QString("%1 %2").arg(fs,0,'f',settings->appearance.digits).arg(UNIT_NAME[u_fs]);
	ms_text=QString("%1 %2").arg(ms,0,'f',settings->appearance.digits).arg(UNIT_NAME[u_ms]);
	us_text=QString("%1 %2").arg(us,0,'f',settings->appearance.digits).arg(UNIT_NAME[u_us]);
	
}

void Device::get_free_space_text(QString &fs,QString &us,QString &ms)
{
	fs=fs_text;
	us=us_text;
	ms=ms_text;
}

QString Device::get_tooltip()
{
	if (!settings->initialized)
		return QString();
	
	QString tooltip=QString("<font color='%1'><table><tr><td colspan='2' align='center'><font color='%9' size='+1'><B><I>%2</I></B></font></td></tr><HR><tr><td>Device is currently</td><td> <font color='%3'>%4</font></td></tr><tr><td>Device name </td><td> <font color='%5'>%6</font></td></tr><tr><td>Mount point</td><td> <font color='%7'>%8</font></td></tr>")
 							.arg(settings->appearance.colors.at(ToolTipText).name())
							.arg(get_nick_name())
							.arg(settings->appearance.colors.at(ToolTipStatus).name())
							.arg(MOUNT_STATE_NAME[get_state()])
							.arg(settings->appearance.colors.at(ToolTipName).name())
 							.arg(get_dev_name())
							.arg(settings->appearance.colors.at(ToolTipMountPoint).name())
 							.arg(get_mount_point())
							.arg(settings->appearance.colors.at(ToolTipNickName).name());
	if (is_show_free_space() && (get_state()==Mounted))
	{
		QString fs,us,ms;
		get_free_space_text(fs,us,ms);
		tooltip+=QString("<tr><td>Free space</td><td><font color='%1'>%2 (%3%)</font></td></tr><tr><td>Used space</td><td><font color='%4'>%5 (%6%)</font></td></tr><tr><td>Device capacity</td><td><font color='%7'>%8</font></td></tr>")
							.arg(settings->appearance.colors.at(ToolTipFreeSpace).name()).arg(fs)						
							.arg(((float)get_free_space())*100.0/get_max_space(),0,'f',settings->appearance.digits)
							.arg(settings->appearance.colors.at(ToolTipUsedSpace).name()).arg(us)						
							.arg(100.0-((float)get_free_space())*100.0/get_max_space(),0,'f',settings->appearance.digits)
							.arg(settings->appearance.colors.at(ToolTipMaxSpace).name()).arg(ms);
	}
	return tooltip+"</table></font>";
}

/////////////////////////////////
void Device::set_type (const DeviceType &t)
{
		structure.type=t;
}

DeviceType Device::get_type() const
{
	return structure.type;
}

void Device::guess_type (const QString &S , DeviceType &type , OperatingSystem &OS) const
{
		// try to guess the OS and physical device types according to the given fstab entries
		if ((S.indexOf("ext")>=0) || (S.indexOf("reiser")>=0))
		{
			type=HardDisk;
			OS=Linux;
		} 
		else if ((S.indexOf("vfat")>=0) || (S.indexOf("msdos")>=0) || (S.indexOf("ntfs")>=0))
		{
			type=HardDisk;
			OS=Windows;
		}
		else if ((S.indexOf("iso9")>=0) || (S.indexOf("udf")>=0))
		{
			type=CDRom;
			OS=Undefined;
		}
		else if (S.indexOf("nfs")>=0)
		{
			type=NetDisk;
			OS=Linux;
		}
		else if (S.indexOf("smbfs")>=0)
		{
			type=NetDisk;
			OS=Windows;
		}
		else if (S.indexOf("fuse")>=0)
		{
			type=FUSE_FS;
			OS=Linux;
			if (get_dev_name().indexOf("sshfs")>=0)
				type=SSH_FS;
		}
		else
		{
			type=Unknown;
			OS=Undefined;
		}
	
}

//////////////////// Uncommented plain self-explanatory boring stuff
QString Device::get_load_command							() const
{
	return structure.load_command;
}

QString	Device::get_eject_command							() const
{
	return structure.eject_command;
}

void Device::set_load_command(const QString &S)
{
		structure.load_command=S;
}

void Device::set_eject_command(const QString &S)
{
		structure.eject_command=S;
}


QString Device::get_dev_name () const
{
		return structure.dev_name;
}

void Device::set_dev_name						(const QString &S)
{
		structure.dev_name=S;
}

/////////////////////////////////////////
void Device::set_state(const MountState &st)
{
		if (state!=st)
		{
			state=st;
		}
}

////////////////////////////////////////

QString	Device::get_mount_point () const
{
		return structure.mount_point;
}

void Device::set_mount_point					(const QString &S)
{
		structure.mount_point=S;
}

////////////////////////////////////////
QString	Device::get_nick_name () const
{
		return structure.nick_name;
}

void Device::set_nick_name					(const QString &S)
{
		structure.nick_name=S;
}

////////////////////////////////////////
void Device::set_free_space								(const quint64 &f)
{
	free_space=f;
	refresh_free_space_text();
}

quint64	Device::get_free_space								() const
{
	return free_space;
}

////////////////////////////////////////
void	Device::set_max_space									(const quint64 &f)
{
	max_space=f;
	refresh_free_space_text();
}

quint64	Device::get_max_space									() const
{
	return max_space;
}
////////////////////////////////////////

void Device::set_show_free_space (const bool &b)
{
	structure.show_free_space=b;
}

bool Device::is_show_free_space() const
{
	return structure.show_free_space;
}
////////////////////////////////////////

QString	Device::get_icon_name () const
{
	return structure.icon_name;
}

void	Device::set_icon_name									(const QString&s)
{
		structure.icon_name=s;
}
////////////////////////////////////////

QString	Device::get_OS_icon_name							() const
{
		return structure.OS_icon_name;
}

void	Device::set_OS_icon_name							(const QString&s)
{
		structure.OS_icon_name=s;
}
////////////////////////////////////////

void	Device::set_OS												(const OperatingSystem&s)
{
		structure.OS=s;
}

OperatingSystem	Device::get_OS												() const
{
		return structure.OS;
}

////////////////////////////////////////

void Device::set_sudo(const bool &b)
{
		structure.sudo=b;
}

bool Device::is_sudo() const
{
		return structure.sudo;
}

////////////////////////////////
MountState Device::get_state() const
{
		return state;
}

bool	Device::is_managed() const
{
		return structure.managed;
}

void	Device::set_managed(const bool &m)
{
		structure.managed=m;	
}

void Device::stop_managing()
{
		set_managed(FALSE);
}

/////////////////////////////////////////////////////////
DeviceInfo	Device::get_info											() const
{
	return structure;
}

void Device::set_info											(const DeviceInfo &inf)
{
	structure=inf;
	set_nick_name(inf.nick_name);
	set_type(inf.type);
	set_icon_name(inf.icon_name);
	set_OS(inf.OS);
	set_OS_icon_name(inf.OS_icon_name);
 	set_managed(inf.managed);
}

////////////////////////////////////////



Device::~Device()
{	
  		delete command;			
}

#include "device.moc"