/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
#ifndef DEVICE_HEADER
#define DEVICE_HEADER
/////////////////////
#include <QObject>
#include <KProcess>
//////////////////////
#include "settings.h"
//////////////////////
class	SettingsStructure;

/////////////////////////////////
// Different Operating Systems //
/////////////////////////////////
#define N_OS_TYPE 5
typedef enum OperatingSystem
{
	Undefined,
	Linux,
	Windows,
	MacOS,
	Other
};

static const char* OS_NAME[]=
{
	"Undefined / Don't show",
	"Linux",
	"Window$",
	"MacO$",
	"Other"
};

static const char* OS_ICON[]=
{
	"",
	"os_linux",
	"os_winxp",
	"os_macos",
	""
};

/////////////////////////////
//  Different devices type //
/////////////////////////////
#define N_DEVICE_TYPE	6
typedef enum DeviceType
{
	Unknown,
	HardDisk,
	NetDisk,
	CDRom,
	SSH_FS,
	FUSE_FS
};

static const char* DEVICE_TYPE_NAME[]=
{
	"?",
	"Hard disk",
	"Network device",
	"CD-Rom",
	"SSH filesystem",
	"FUSE filesystem"
};

static const char* DEVICE_TYPE_ICON[]=
{
	"?",
	"drive-harddisk",
	"network-server",
	"drive-optical",
	"security-high",
	"application-x-tarz"
};

static const char* DEVICE_LOAD_ACTION[]=
{
	"",
	"",
	"",
	"eject -t %1",
	"",
	""
};

static const char* DEVICE_EJECT_ACTION[]=
{
	"",
	"",
	"",
	"eject %1",
	"",
	""
};
////////////////////////////////////////
// Different mount state for a device //
////////////////////////////////////////
typedef enum MountState
{
	Unmounted = 0,
	Mounted = 1
};

static const char* MOUNT_STATE_NAME[]=
{
	"unmounted",
	"mounted"
};


/////////////////////////////////////
//  Stores the device informations //
/////////////////////////////////////
struct DeviceInfo
{
	// the physical name (as provided in fstab)
	QString								dev_name;
	// the mount point (as provided in fstab)
	QString								mount_point;
	// this is the displayed name of the device
	QString								nick_name;
	// the device type (e.g. cd-rom, hard-disk ...)
	DeviceType						type;	
	// custom device type icon
	QString								icon_name;
	// the device OS
	OperatingSystem				OS;
	// custom OS icon
	QString								OS_icon_name;
	// is the device managed/displayed by the applet?
	bool									managed;
	// does the applet show space-related information?
	bool									show_free_space;
	// does the device need to be mounted as root?
	bool									sudo;
	// commands to load/eject the device, if relevant
	QString								load_command;
	QString								eject_command;
};


////////////////////////////////////////////////////////////
// This class provides the abstract part of the device, 
// containing all the data and state informations

class Device : public QObject
{
	Q_OBJECT
	public:
															Device												(const QString& dev,
																														const QString& mt,
																														SettingsStructure *set);
															~Device												();
				// mounts the device if it is unmounted and vice-versa
				void									toggle_mount									();
				// interfaces mount state information
				void									set_state											(const MountState &);
				MountState						get_state											() const;
				// interfaces device information
				DeviceInfo						get_info											() const;
				void									set_info											(const DeviceInfo &);
				// interfaces device management information
				bool									is_managed										() const;
				void									set_managed										(const bool &);
				// interfaces device physical name information
				void									set_dev_name									(const QString &);
				QString								get_dev_name									() const;
				// interfaces mount point information
				void									set_mount_point								(const QString &);
				QString								get_mount_point								() const;
				// interfaces nickname information
				void									set_nick_name									(const QString &);
				QString								get_nick_name									() const;
				// interfaces custom icon information
				QString								get_icon_name									() const;	
				void									set_icon_name									(const QString&);	
				// interfaces custom OS icon information
				QString								get_OS_icon_name							() const;	
				void									set_OS_icon_name							(const QString&);	
				// interfaces OS information
				void									set_OS												(const OperatingSystem&);
				OperatingSystem				get_OS												() const;
				// actions to perform when the user invokes the load or eject command
				void									set_load_command							(const QString &);
				QString								get_load_command							() const;
				void									set_eject_command							(const QString &);				
				QString								get_eject_command							() const;
				// interfaces device type information
				void									set_type											(const DeviceType &);
				DeviceType						get_type											() const;
				// try to guess the device and OS types
				// according to the provided fstab entries
				void									guess_type										(const QString &S,
																														 DeviceType &type ,
																														 OperatingSystem &OS) const;
				// interfaces displaying free space information
				void									set_show_free_space						(const bool&);
				bool									is_show_free_space						() const;
				// interfaces mount as root information
				void									set_sudo											(const bool&);
				bool									is_sudo												() const;
				// interfaces free space information
				void									set_free_space								(const quint64 &);
				quint64								get_free_space								() const;
				// interfaces max space type information
				void									set_max_space									(const quint64 &);
				quint64								get_max_space									() const;
				// a pointer to the applet settings structure
				SettingsStructure			*settings;								
				// transform free/used/max space into QString and their associated units
				void									get_free_space_text						(QString &fs,
																														 QString &us,
																														 QString &ms);				
				// called when free spaceinformations changes
				void 									refresh_free_space_text				();
				// this is a RichText describing all informaions of the device, to be dsiplayed as tooltips		
				QString								get_tooltip										();
				
	private:									
				// the mount state of the device
				MountState						state;		
				// the configuration information of the device
				DeviceInfo						structure;
				// the Process managing the mount/umount or load/eject command
				KProcess*							command;
				// to store the latest executed command
				QString								current_command;
			// information about the size of the device
				quint64								free_space;
				quint64								max_space;
				// stores QString values of the (free/used/max) sizes and their units
				// modified by refresh_free_space_text()
				QString								fs_text;
				QString								us_text;
				QString								ms_text;
				
				
	public slots:
				// called to sync the data and free space informations
				void									some_data_changed							();
				// called when the user requests to stop managing this device
				void									stop_managing									();
				// executes the load command
				void									do_load												(const bool &);
				// launches the Process command with the corresponding mount state
    		void									do_mount											(const MountState&);
				// executes the file manager
				void									launch_file_manager						();
								
				    		
	private slots:				
				// called when the QProcess command is done
				void									command_finished							(int exitCode, 
																														QProcess::ExitStatus exitStatus);
				// called when the Process command fails				
				void									command_failed								(QProcess::ProcessError);
				// the Process requires some input from the user
				void									enter_input										();		
				
	signals:
				// emited after a mount command to sync device sizes information
				void									SIG_refresh_mtab							();
};

#endif
