/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsSceneMoveEvent>
#include <QMenu>
#include <QAction>
//////////////////////////////////////
#include "icon_widget.h"
#include "settings.h"
#include "device.h"
#include "mountoid_view.h"
#include "icon_view.h"
//////////////////////////////////


/////////////////////////////////////////////////////////////
/////         																					/////
/////     Class IconWidget															/////
/////         																					/////
/////////////////////////////////////////////////////////////

IconWidget::IconWidget(Device* par, IconViewWidget* vpar ) : QGraphicsItem()
{
		par_dev=par;
		view=vpar;
		width=1.0;
		is_clicked=FALSE;
}


void IconWidget::paint(QPainter *painter,const QStyleOptionGraphicsItem *option,QWidget *par)
{
	// this is the working rectangle
	QRect rect = boundingRect().toRect();
	
	
	// if the icon is pressed down and not moving, draw it shrinked, else draw it normal
	if (is_clicked && par_dev->get_state()==Mounted && !view->is_moving)
		dev_icon.paint(painter,rect.adjusted(5,5,-5,-5), Qt::AlignCenter,IconState[QIcon::Selected],QIcon::On);
	else
		dev_icon.paint(painter,rect, Qt::AlignCenter,IconState[par_dev->get_state()],QIcon::On);
	
	// if the device has defined OS then print its icon in the midle of the device type icon
	if (par_dev->get_OS())
		OS_icon.paint(painter,QRect(QPoint(width/2-par_dev->settings->appearance.OS_size/2,width/2-par_dev->settings->appearance.OS_size/2),QSize(par_dev->settings->appearance.OS_size,par_dev->settings->appearance.OS_size)));
	
	// displays the used space bar at the bottom of the icon if relevant
	if (par_dev->is_show_free_space()  && (par_dev->get_state()==Mounted))
	{
		float tx=1.0-((float)par_dev->get_free_space())/par_dev->get_max_space();
		float rate=tx;
		QColor c0,c1;
		// we want to make an affine coloring of the bar
		// between the three states Empty, HalfFull and Full
		// we first determine in which segment we are
		if (tx<=0.5)	// between Emty and HalfFull
		{
			c0=par_dev->settings->appearance.colors.at(EmptyDevice);
			c1=par_dev->settings->appearance.colors.at(HalfFullDevice);
		}
		else 	// between HalfFull and Full
		{
			c1=par_dev->settings->appearance.colors.at(FullDevice);
			c0=par_dev->settings->appearance.colors.at(HalfFullDevice);
			tx=tx-0.5;
		}
		tx*=2.0;
		float xt = 1.0-tx;
		
		// blend the colors
		QColor br_c;
		br_c.setRedF(c0.redF()*xt + c1.redF()*tx);
		br_c.setGreenF(c0.greenF()*xt + c1.greenF()*tx);
		br_c.setBlueF(c0.blueF()*xt + c1.blueF()*tx );
		br_c.setAlphaF(0.5);
				
		// and finally draw the bar
		QBrush brush(br_c);
		painter->setBrush(brush);
		painter->setPen(Qt::NoPen);
		painter->drawRect(QRectF(QPointF(0.0,width-8.0),QSizeF((width-8.0)*rate,8.0)));
		
		painter->setBrush(Qt::NoBrush);
		painter->setPen(Qt::white);
		painter->drawRect(QRectF(QPointF(0.0,width-8.0),QSizeF(width-8.0,8.0)));
	}
}

//////////////////// Uncommented plain self-explanatory boring stuff

void IconWidget::contextMenuEvent	(QGraphicsSceneContextMenuEvent *ev)
{
		view->context_menu(ev,par_dev);		
}

void		IconWidget::mousePressEvent				(QGraphicsSceneMouseEvent *ev)
{
		if (ev->button() & Qt::LeftButton )
		{
			is_clicked=TRUE;
			ev->accept();
			update();
		}
}

void		IconWidget::mouseReleaseEvent			(QGraphicsSceneMouseEvent *ev)
{
		if ((is_clicked) && (ev->button() & Qt::LeftButton ))
		{
			if (view->is_moving)
			{
				view->is_moving=FALSE;
				view->refresh_size();
			}
			else
			{
				view->icon_clicked(par_dev);
			}
			is_clicked=FALSE;
			setCursor(Qt::ArrowCursor);
			ev->accept();
			update();
		}
}


void IconWidget::mouseMoveEvent(QGraphicsSceneMouseEvent* ev)
{
	if (is_clicked)
	{
		setCursor(Qt::SizeAllCursor);
		QPointF trans = mapToParent(mapFromScene(ev->scenePos()));
		if (view->boundingRect().contains(trans))
		{
			if (view->get_settings()->appearance.display_mode==StackedHorizontally)
				trans.setY(pos().y());
			else
				trans.setX(pos().x());
			setPos(trans);
		}
		view->icon_moving(this);
		ev->accept();
		update();
	}
}


void IconWidget::synchronize_tooltip()
{
	setToolTip(par_dev->get_tooltip());
}

void IconWidget::set_width(const float &w)
{
		width=w;
		update();
}


float IconWidget::get_width() const
{
	return width;
}


QRectF IconWidget::boundingRect() const
{
	return QRectF(QPointF(),QSizeF(width,width));
}
		
void IconWidget::set_icon (const QString& ico)
{
		dev_icon=KIcon(ico);
		update();
}
	
void IconWidget::set_icon_OS (const QString& ico)
{
		OS_icon=KIcon(ico);
		update();
}
	
void IconWidget::data_changed()
{
		synchronize_tooltip();
		update();
}
